#encoding = utf8
import copy
import logging
import queue
import time
from queue import Queue
from threading import Thread, Event

import cv2
import numpy as np

from human_handler import AudioHandler


class HumanRender(AudioHandler):
    def __init__(self, context, handler):
        super().__init__(context, handler)

        self._queue = Queue(context.batch_size * 2)
        self._audio_render = None
        self._image_render = None

        self._exit_event = Event()
        self._thread = Thread(target=self._on_run)
        self._exit_event.set()
        self._thread.start()

    def _on_run(self):
        logging.info('human render run')
        while self._exit_event.is_set():
            self._run_step()
            time.sleep(0.002)

        logging.info('human render exit')

    def _run_step(self):
        try:
            res_frame, idx, audio_frames = self._queue.get(block=True, timeout=.002)
        except queue.Empty:
            # print('render queue.Empty:')
            return None
        if audio_frames[0][1] != 0 and audio_frames[1][1] != 0:
            combine_frame = self._context.frame_list_cycle[idx]
        else:
            bbox = self._context.coord_list_cycle[idx]
            combine_frame = copy.deepcopy(self._context.frame_list_cycle[idx])
            y1, y2, x1, x2 = bbox
            try:
                res_frame = cv2.resize(res_frame.astype(np.uint8), (x2 - x1, y2 - y1))
            except:
                return
            # combine_frame = get_image(ori_frame,res_frame,bbox)
            # t=time.perf_counter()
            combine_frame[y1:y2, x1:x2] = res_frame

        image = combine_frame
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)

        if self._image_render is not None:
            self._image_render.on_render(image)

        for audio_frame in audio_frames:
            frame, type_ = audio_frame
            frame = (frame * 32767).astype(np.int16)
            if self._audio_render is not None:
                self._audio_render.write(frame.tobytes(), int(frame.shape[0]*2))
            # new_frame = AudioFrame(format='s16', layout='mono', samples=frame.shape[0])
            # new_frame.planes[0].update(frame.tobytes())
            # new_frame.sample_rate = 16000

    def set_audio_render(self, render):
        self._audio_render = render

    def set_image_render(self, render):
        self._image_render = render

    def on_handle(self, stream, index):
        self._queue.put(stream)

    def stop(self):
        self._exit_event.clear()
        self._thread.join()
